export default { init };

function init() {
  checkVersion();
  setInterval(checkVersion, 30000);
}

function checkVersion() {
  const localVersion = process.env.REACT_APP_VERSION;
  if (!localVersion) {
    console.debug(
      '[cache-updater] you have to set REACT_APP_VERSION=$npm_package_version in .env file'
    );
    return;
  }
  console.debug('[cache-updater] local version: ' + localVersion);
  if (process.env.NODE_ENV != 'development') {
    console.debug('[cache-updater] checking server version');
    fetch('meta.json?' + new Date().getTime())
      .then((response) => response.json())
      .then((meta) => {
        if (!meta || !meta.version) {
          console.debug('[cache-updater] meta.json file missing');
          return;
        }
        const serverVersion = meta.version;
        console.debug('[cache-updater] server version: ' + serverVersion);
        if (semverGreaterThan(serverVersion, localVersion)) {
          console.debug('[cache-updater] updating');
          forceSWupdate();
        } else {
          console.debug('[cache-updater] nothing to update');
        }
      });
  }
}

function forceSWupdate() {
  if ('serviceWorker' in navigator) {
    navigator.serviceWorker.getRegistrations().then(function (registrations) {
      for (let registration of registrations) {
        console.log('updating', JSON.stringify(registration));
        registration.update();

        caches.keys().then(function (names) {
          for (let name of names) caches.delete(name);
        });
      }
    });
  }
}

function semverGreaterThan(versionA, versionB) {
  const versionsA = versionA.split(/\./g);

  const versionsB = versionB.split(/\./g);
  while (versionsA.length || versionsB.length) {
    const a = Number(versionsA.shift());

    const b = Number(versionsB.shift());
    // eslint-disable-next-line no-continue
    if (a === b) continue;
    // eslint-disable-next-line no-restricted-globals
    return a > b || isNaN(b);
  }
  return false;
}
